@@ -33,9 +33,9 @@ class QPushButton;
 class FileParameter : public AbstractParameter {
   Q_OBJECT
 public:
-  FileParameter(QObject * parent = 0);
+  FileParameter(QObject * parent = nullptr);
   ~FileParameter();
-  void addTo(QWidget *, int row) override;
+  bool addTo(QWidget *, int row) override;
   QString textValue() const override;
   QString unquotedTextValue() const override;
   void setValue(const QString & value) override;
