@@ -29,18 +29,18 @@
 #include "Common.h"
 #include "DialogSettings.h"
 
-SeparatorParameter::SeparatorParameter(QObject * parent) : AbstractParameter(parent, false), _frame(0) {}
+SeparatorParameter::SeparatorParameter(QObject * parent) : AbstractParameter(parent, false), _frame(nullptr) {}
 
 SeparatorParameter::~SeparatorParameter()
 {
   delete _frame;
 }
 
-void SeparatorParameter::addTo(QWidget * widget, int row)
+bool SeparatorParameter::addTo(QWidget * widget, int row)
 {
-  QGridLayout * grid = dynamic_cast<QGridLayout *>(widget->layout());
-  if (!grid)
-    return;
+  _grid = dynamic_cast<QGridLayout *>(widget->layout());
+  Q_ASSERT_X(_grid, __PRETTY_FUNCTION__, "No grid layout in widget");
+  _row = row;
   delete _frame;
   _frame = new QFrame(widget);
   QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
@@ -53,7 +53,8 @@ void SeparatorParameter::addTo(QWidget * widget, int row)
   if (DialogSettings::darkThemeEnabled()) {
     _frame->setStyleSheet("QFrame{ border-top: 0px none #a0a0a0; border-bottom: 2px solid rgb(160,160,160);}");
   }
-  grid->addWidget(_frame, row, 0, 1, 3);
+  _grid->addWidget(_frame, row, 0, 1, 3);
+  return true;
 }
 
 QString SeparatorParameter::textValue() const
